// DrawSDIDoc.cpp : implementation of the CDrawSDIDoc class
//

#include "stdafx.h"
#include "DrawSDI.h"

#include "DrawSDIDoc.h"
#include ".\drawsdidoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#include "MainFrm.h"
// CDrawSDIDoc

IMPLEMENT_DYNCREATE(CDrawSDIDoc, CDocument)

BEGIN_MESSAGE_MAP(CDrawSDIDoc, CDocument)
	ON_COMMAND_RANGE(ID_COLOR_BLACK, ID_COLOR_WHITE, OnColorCommand)
	ON_UPDATE_COMMAND_UI_RANGE(ID_COLOR_BLACK, ID_COLOR_WHITE, OnUpdateColorUI)
	ON_COMMAND_RANGE(ID_WIDTH_1, ID_WIDTH_32, OnWidthCommand)
	ON_UPDATE_COMMAND_UI_RANGE(ID_WIDTH_1, ID_WIDTH_32, OnUpdateWidthUI)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COLOR, OnUpdateIndicatorColor)
END_MESSAGE_MAP()

const COLORREF CDrawSDIDoc::m_Colors[] = {
	RGB(0, 0, 0),		// Black - 
	RGB(0, 0, 255),		// Blue - 
	RGB(0, 255, 0),		// Green - 
	RGB(0, 255, 255),	// Cyan - 
	RGB(255, 0, 0),		// Red - 
	RGB(255, 0, 255),	// Magenta - 
	RGB(255, 255, 0),	// Yellow - 
	RGB(255, 255, 255)	// White - 
};


// CDrawSDIDoc construction/destruction

CDrawSDIDoc::CDrawSDIDoc()
{
	// TODO: add one-time construction code here

}

CDrawSDIDoc::~CDrawSDIDoc()
{
}

BOOL CDrawSDIDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	//    
	//   1
	m_iColor = 0;
	m_iWidth = 1;
	UpdateComboBox(0);

	return TRUE;
}

// CDrawSDIDoc serialization

void CDrawSDIDoc::Serialize(CArchive& ar)
{
	//  - 
	//     
	m_Lines.Serialize(ar);
}


// CDrawSDIDoc diagnostics

#ifdef _DEBUG
void CDrawSDIDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDrawSDIDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

//  
MyLine* CDrawSDIDoc::AddLine(CPoint pointBegin, CPoint pointEnd)
{
	MyLine* pLine = NULL; //    

	try // 
	{
		//   -  
		pLine = new MyLine(pointBegin, pointEnd); // 
		//        
		pLine->Set(m_Colors[m_iColor], m_iWidth);
		//        
		m_Lines.Add(pLine);
		//    
		SetModifiedFlag();
	}
	catch(CMemoryException* pErr) //    
	{
		//  
		AfxMessageBox(" ", MB_ICONSTOP | MB_OK);
		//  -,    
		if(pLine){
			delete pLine;
			pLine = NULL;
		}

		//    
		pErr->Delete();
	}

	return pLine; //    -
}

//   
int CDrawSDIDoc::GetLineCount(void)
{
	//     
	return (int)m_Lines.GetCount();
}

//    -  
MyLine* CDrawSDIDoc::GetLine(int index)
{
	//    -,  
	//      CObArray m_Lines
	return (MyLine*)m_Lines[index];
}

void CDrawSDIDoc::DeleteContents()
{
	//      
	int count = (int)m_Lines.GetCount();
	
	//     
	try 
	{
		if(count){ //   

			//       
			for(int i = 0; i < count; i++)
				delete (MyLine*)m_Lines.GetAt(i);

			//     
			m_Lines.RemoveAll();
		}
	}
	//     
	catch(CMemoryException* pErr) 
	{
		//     
		pErr->ReportError();

		//  ,  
		pErr->Delete();
	}

	CDocument::DeleteContents();
}

//        Color
void CDrawSDIDoc::OnColorCommand(UINT nID)
{
	m_iColor = nID - ID_COLOR_BLACK;
}

//        Color
void CDrawSDIDoc::OnUpdateColorUI(CCmdUI *pCmdUI)
{
	//    (  TRUE)  
	pCmdUI->Enable(); 
	bool check = pCmdUI->m_nID == m_iColor + ID_COLOR_BLACK;
	//   ,  
	//     
	pCmdUI->SetCheck(check ? TRUE : FALSE);
}

//        Width
void CDrawSDIDoc::OnWidthCommand(UINT nID)
{
	switch(nID){
		case ID_WIDTH_16: m_iWidth = 16; break;
		case ID_WIDTH_24: m_iWidth = 24; break;
		case ID_WIDTH_32: m_iWidth = 32; break;
		default: m_iWidth = nID - ID_WIDTH_1 + 1;
	}
	UpdateComboBox(nID - ID_WIDTH_1);
}

//        Width
void CDrawSDIDoc::OnUpdateWidthUI(CCmdUI *pCmdUI)
{
	//    (  TRUE)  
	pCmdUI->Enable(); 

	bool check;
	int pos = 0;
	if(m_iWidth <= 8)
		pos = m_iWidth - 1; //0-7
	else if(m_iWidth == 16)
		pos = m_iWidth - 8;	//8
	else if(m_iWidth == 24)
		pos = m_iWidth - 15;//9
	else if(m_iWidth == 32)
		pos = m_iWidth - 22;//10
	//   ,  
	//     
	check = pCmdUI->m_nID == pos + ID_WIDTH_1;
	pCmdUI->SetCheck(check ? TRUE : FALSE);
}

//   
void CDrawSDIDoc::SetWidth(int nWidth)
{
	m_iWidth = nWidth;
}

//    
void CDrawSDIDoc::UpdateComboBox(int iIndex)
{
	//      Z- 
	POSITION pos = GetFirstViewPosition();
	if(!pos) //   ,  
		return;

	//      ( )
	CView* pView = GetNextView(pos);
	if(!pView) //   ,  
		return;

	//        
	CMainFrame* pFrame = (CMainFrame*)pView->GetTopLevelFrame();
	if(!pFrame) //   ,  
		return;

	//      
	pFrame->m_ctrlWidth.SetCurSel(iIndex);
}

//   
void CDrawSDIDoc::OnUpdateIndicatorColor(CCmdUI* pCmdUI)
{
	char* str[] = {"", 
					"",
					"",
					"",
					"",
					"",
					"",
					""	
					};
	pCmdUI->Enable(); //   TRUE - 
	pCmdUI->SetText((LPCTSTR)str[m_iColor]);
}
